Class: Norn::Modes::Chat
Constant Summary
Constants inherited
from Norn::Mode
Norn::Mode::ABSTRACT_METHODS
Instance Attribute Summary
Attributes inherited from Norn::Mode
#input, #messages, #output
Instance Method Summary
collapse
Methods inherited from Norn::Mode
#compile_system_prompt, #execute_tool, #initialize, #start
Constructor Details
This class inherits a constructor from Norn::Mode
Instance Method Details
#allowed_capabilities ⇒ Object
11
12
13
|
# File 'lib/norn/modes/chat.rb', line 11
def allowed_capabilities
[:sys_read, :vcs_read]
end
|
#banner_name ⇒ Object
15
16
17
|
# File 'lib/norn/modes/chat.rb', line 15
def banner_name
"Norn MVP Chatbot"
end
|
#instructions ⇒ Object
19
20
21
22
23
24
|
# File 'lib/norn/modes/chat.rb', line 19
def instructions
"You are in Chat Mode. In this mode, you have access to a set of safe read-only tools " \
"(file_read, glob, grep) that you can use to inspect the codebase and answer the user's questions. " \
"Use these tools whenever you need live code context to help the user. " \
"Focus on helping the user through helpful conversation and code exploration only."
end
|
#interactive? ⇒ Boolean
7
8
9
|
# File 'lib/norn/modes/chat.rb', line 7
def interactive?
true
end
|